#!/bin/bash

E_BASE_REPO="http://svn.enlightenment.org/svn/e"
E_TRUNK_REPO="$E_BASE_REPO/trunk"
E_PYTHON_SUFIX="BINDINGS/python"
E_PYTHON_REPO="$E_TRUNK_REPO/$E_PYTHON_SUFIX"
E_PROTO_REPO="$E_TRUNK_REPO/PROTO"

[ -z $OUTPUT_FILE ] && OUTPUT_FILE="/dev/null"
[ -z $E_SVN_REV ] && E_SVN_REV="50928"

TMP_DIR="maemo-efl-deb-src-$E_SVN_REV"
VERSIONS_DIR="$TMP_DIR/versions"

# C Modules
ALL_MODULES=("eina"
             "eet"
             "evas"
             "ecore"
             "embryo"
             "edje"
             "e_dbus"
             "emotion"
             "epdf"
             "ethumb"
             "libeweather"
             "elementary"
             "efl-core")

# Python Bindings
ALL_MODULES+=("python-evas"
              "python-ecore"
              "python-edje"
              "python-e_dbus"
              "python-ethumb"
              "python-efl_utils"
              "python-elementary"
              "python-efl-core")

MAEMO_REV="maemo1"
MAEMO_REPO="extras-devel"

function exit_error(){
    echo " ERROR" && exit 1
}

function init(){
    if [ -d $TMP_DIR ]; then
		echo -n "Cleaning up ..."
		rm -rf $TMP_DIR || exit_error
        echo " OK"
    fi

	echo -n "Creating build dir ..."
    mkdir -p $TMP_DIR || exit_error
    echo " OK"
}

function checkout_repositories(){
    echo "Checking out repositories ..."
    for module in ${ALL_MODULES[@]};do
        echo -n "	$module"

        case $module in
            elementary)
                REPO="$E_TRUNK_REPO/TMP/st"
                ;;
            epdf|libeweather)
                REPO="$E_PROTO_REPO"
                ;;
            *efl-core)
                cp -r $module $TMP_DIR >> $OUTPUT_FILE 2>&1 && echo " ... OK" || exit_error
                continue
                ;;
            python-*)
                REPO="$E_PYTHON_REPO"
                ;;
            *)
                REPO=$E_TRUNK_REPO
                ;;
        esac

		REV="-r$E_SVN_REV "

        echo -n " ."
        svn co $REV $REPO/$module $TMP_DIR/$module >> $OUTPUT_FILE 2>&1 || exit_error
        echo -n "."
        rm -rf $TMP_DIR/$module/debian
        echo -n "."
        cp -r $module/debian $TMP_DIR/$module/debian >> $OUTPUT_FILE 2>&1 || exit_error
        echo " OK"
    done

	echo -n "Cleaning up repositories ..."
	rm -rf `find $TMP_DIR -name .svn -type d`
	rm -rf `find $TMP_DIR -name .git -type d`
    echo " OK"
}

function c_module_version() {
    local mod=$1

    [ -z $mod ] && exit_error

    VERSION=`grep AC_INIT $TMP_DIR/$mod/configure.ac | cut -d[ -f3 | cut -d] -f1 || exit_error`
    if [ $VERSION = "v_ver" ]; then
        MAJOR=`grep v_maj $TMP_DIR/$mod/configure.ac | xargs | cut -d[ -f3 | cut -d] -f1 || exit_error`
        MINOR=`grep v_min $TMP_DIR/$mod/configure.ac | xargs | cut -d[ -f3 | cut -d] -f1 || exit_error`
        MICRO=`grep v_mic $TMP_DIR/$mod/configure.ac | xargs | cut -d[ -f3 | cut -d] -f1 || exit_error`
        if [ -z $MAJOR ] || [ -z $MINOR ] || [ -z $MICRO ]; then
            exit_error
        fi

        VERSION=$MAJOR.$MINOR.$MICRO
    fi
}

function update_changelogs(){
	echo "Updating changelogs ..."
    for module in ${ALL_MODULES[@]};do
        echo -n "	$module ."

        case $module in
            *efl-core)
                c_module_version "evas"
                ;;
            python-*)
                VERSION=`grep version $TMP_DIR/$module/setup.py | cut -d\' -f2 || exit_error`
                ;;
            *)
                c_module_version "$module"
                ;;
        esac

        echo -n "."
        if [ -z $VERSION ] || [ $VERSION = "v_ver" ]; then
            exit_error
        fi

        echo -n "."
        dch --force-distribution -b -c $TMP_DIR/$module/debian/changelog -v $VERSION.$E_SVN_REV-$MAEMO_REV -D $MAEMO_REPO "SVN snapshot release $E_SVN_REV" >> $OUTPUT_FILE 2>&1 || exit_error
        echo " `head -n1 $TMP_DIR/$module/debian/changelog`"
    done
}

function gen_versions(){
    if [ ! -d $VERSIONS_DIR ]; then
        echo -n "Creating versions dir $VERSIONS_DIR ..."
        mkdir -p $VERSIONS_DIR || exit_error
        echo " OK"
    fi

    echo "Generating version files ..."
    for module in ${ALL_MODULES[@]};do
        VERSION=`dpkg-parsechangelog -l$TMP_DIR/$module/debian/changelog | grep Version: | cut -d" " -f2 || exit_error`
        echo "	$module ... $VERSION"
        echo "$VERSION" > $VERSIONS_DIR/"$module"_version
    done
}

gen_control(){
    echo "Generating control files ..."
    for module in ${ALL_MODULES[@]}; do
        echo -n "	$module ..."
        if [ -e $TMP_DIR/$module/debian/control.in ]; then
            TO_REPLACE=`grep -o -E '@([A-Z]|[0-9])+((-|_)*([A-Z]|[0-9]))*_VERSION@' $TMP_DIR/$module/debian/control.in | cut -d @ -f 2 | sort -u`
            cp $TMP_DIR/$module/debian/control.in $TMP_DIR/$module/debian/control.temp

            for replacement in $TO_REPLACE; do
                replacement_lowercase=`echo $replacement | tr "[:upper:]" "[:lower:]"`
                version_file=$VERSIONS_DIR/$replacement_lowercase
                if [ ! -e $version_file ]; then
                    echo -n " Could not find a replacement for $replacement" && exit_error
                fi
                version_number=`cat $version_file`
                echo -n " $replacement: $version_number"
                sed -i s/@$replacement@/$version_number/g $TMP_DIR/$module/debian/control.temp
            done

            mv $TMP_DIR/$module/debian/control.temp $TMP_DIR/$module/debian/control
        fi

        echo " OK"
    done
}

gen_src_packages(){
    echo "Generating source packages ..."
    for module in ${ALL_MODULES[@]}; do
        echo -n "	$module ."
        cd $TMP_DIR/$module || exit_error
        echo -n "."
        dpkg-buildpackage -rfakeroot -us -uc -sa -d -S -I.git -I.gitignore -I.svn -I.svnignore -ICVS -I.cvsignore >> $OUTPUT_FILE 2>&1|| exit_error
        echo -n "."
        cd - >> $OUTPUT_FILE 2>&1 || exit_error
        echo " OK"
    done
}

init
checkout_repositories
update_changelogs
gen_versions
gen_control
gen_src_packages
echo "All done. Packages in $TMP_DIR"
exit 0
